import discord
from cpl_core.logging import LoggerABC

from gismo_core.abc.events.on_member_update_abc import OnMemberUpdateABC
from gismo_core.abc.events.on_ready_abc import OnReadyABC
from gismo_core.abc.module_abc import ModuleABC
from modules.permission.abc.permission_service_abc import PermissionServiceABC
from modules.permission.configuration.permission_settings import PermissionSettings


class Permission(ModuleABC, OnReadyABC, OnMemberUpdateABC):

    def __init__(
        self,
        logger: LoggerABC,
        permission_service: PermissionServiceABC
    ):
        self._logger = logger
        self._permission_service = permission_service

        ModuleABC.__init__(
            self,
            {OnReadyABC: 1, OnMemberUpdateABC: 0},
            PermissionSettings
        )
        self._logger.info(__name__, f'Module {type(self)} loaded')

    async def on_ready(self):
        self._logger.debug(__name__, f'Module {type(self)} started')
        self._permission_service.on_ready()

    async def on_member_update(self, before: discord.Member, after: discord.Member):
        self._logger.debug(__name__, f'Module {type(self)} started')

        if before.roles != after.roles:
            self._permission_service.on_member_update(before, after)
